VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "DraggablePolygon"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public Event VertexMoved()
Public Event PolygonMoved()

Private Const HANDLE_WIDTH = 6
Private Const HANDLE_HALF_WIDTH = HANDLE_WIDTH / 2

Private Declare Function Polygon Lib "gdi32" (ByVal hdc As Long, lpPoint As POINTAPI, ByVal nCount As Long) As Long
Private Type POINTAPI
    X As Long
    Y As Long
End Type

Private m_NumVertices As Long
Private m_Vertices() As POINTAPI

' The index the point is over.
Private m_OverPoint As Integer

' The point the mouse started dragging when
' dragging the polygon.
Private m_DragStartX As Single
Private m_DragStartY As Single

' Used to get events while dragging a vertex.
Private WithEvents m_DragVertexCanvas As PictureBox
Attribute m_DragVertexCanvas.VB_VarHelpID = -1

' Used to get events while dragging the polygon.
Private WithEvents m_DragPolygonCanvas As PictureBox
Attribute m_DragPolygonCanvas.VB_VarHelpID = -1

' Add a new vertex to the polygon.
Public Sub AddVertex(ByVal X As Long, ByVal Y As Long)
    m_NumVertices = m_NumVertices + 1
    ReDim Preserve m_Vertices(1 To m_NumVertices)
    With m_Vertices(m_NumVertices)
        .X = X
        .Y = Y
    End With
End Sub
' Return the distance from a point to a segment.
Private Function DistToSegment(ByVal px As Single, ByVal py As Single, ByVal x1 As Single, ByVal y1 As Single, ByVal x2 As Single, ByVal y2 As Single) As Single
Dim dx As Single
Dim dy As Single
Dim t As Single

    dx = x2 - x1
    dy = y2 - y1
    If dx = 0 And dy = 0 Then
        ' It's a point not a line segment.
        dx = px - x1
        dy = py - y1
        DistToSegment = Sqr(dx * dx + dy * dy)
        Exit Function
    End If

'    t = (dx * (px - X1) + dy * (py - Y1)) / (dx * dx + dy * dy)
    ' This version is simpler.
    t = (px + py - x1 - y1) / (dx + dy)

    If t < 0 Then
        dx = px - x1
        dy = py - y1
    ElseIf t > 1 Then
        dx = px - x2
        dy = py - y2
    Else
        x2 = x1 + t * dx
        y2 = y1 + t * dy
        dx = px - x2
        dy = py - y2
    End If
    DistToSegment = Sqr(dx * dx + dy * dy)
End Function


' Draw the selected vertex.
Public Sub DragVertex(ByVal canvas As PictureBox, ByVal X As Long, ByVal Y As Long)
    ' Set the vertex drag PictureBox so we can
    ' catch its mouse events.
    Set m_DragVertexCanvas = canvas
    m_DragVertexCanvas.DrawMode = vbInvert

    ' Redraw in invert mode.
    Draw m_DragVertexCanvas
End Sub
' Draw the entire polygon.
Public Sub DragPolygon(ByVal canvas As PictureBox, ByVal X As Long, ByVal Y As Long)
    ' Start dragging here.
    m_DragStartX = X
    m_DragStartY = Y
    
    ' Set the polygon drag PictureBox so we can
    ' catch its mouse events.
    Set m_DragPolygonCanvas = canvas
    m_DragPolygonCanvas.DrawMode = vbInvert

    ' Redraw in invert mode.
    Draw m_DragPolygonCanvas
End Sub
' Draw the polygon.
Public Sub Draw(ByVal canvas As PictureBox)
Dim i As Integer

    If m_NumVertices < 1 Then Exit Sub

    ' Draw the polygon.
    canvas.FillStyle = vbFSTransparent
    Polygon canvas.hdc, m_Vertices(1), m_NumVertices

    ' Draw grab handles as white squares with
    ' black edges.
    canvas.FillColor = vbWhite
    canvas.FillStyle = vbFSSolid
    For i = 1 To m_NumVertices
        With m_Vertices(i)
            canvas.Line (.X - HANDLE_HALF_WIDTH, .Y - HANDLE_HALF_WIDTH)- _
                Step(HANDLE_WIDTH, HANDLE_WIDTH), , B
        End With
    Next i
End Sub

' Return True if the point is over a polygon edge.
Public Function IsOverEdge(ByVal X As Single, ByVal Y As Single) As Boolean
Dim x1 As Long
Dim y1 As Long
Dim x2 As Long
Dim y2 As Long

    ' Assume the point is over an edge.
    IsOverEdge = True

    With m_Vertices(m_NumVertices)
        x1 = .X
        y1 = .Y
    End With

    For m_OverPoint = 1 To m_NumVertices
        With m_Vertices(m_OverPoint)
            x2 = .X
            y2 = .Y
        End With

        If DistToSegment(X, Y, x1, y1, x2, y2) < HANDLE_HALF_WIDTH Then
            Exit Function
        End If

        x1 = x2
        y1 = y2
    Next m_OverPoint

    IsOverEdge = False
End Function
' Return True if the point is over a vertex.
Public Function IsOverVertex(ByVal X As Single, ByVal Y As Single) As Boolean
    ' Assume the point is over a vertex.
    IsOverVertex = True

    For m_OverPoint = 1 To m_NumVertices
        With m_Vertices(m_OverPoint)
            If Abs(.X - X) < HANDLE_HALF_WIDTH And _
               Abs(.Y - Y) < HANDLE_HALF_WIDTH _
            Then
                Exit Function
            End If
        End With
    Next m_OverPoint

    IsOverVertex = False
End Function

' Continue dragging the polygon.
Private Sub m_DragPolygonCanvas_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
Dim i As Integer
Dim dx As Single
Dim dy As Single

    ' Erase the previous invert mode image.
    Draw m_DragPolygonCanvas

    ' See how far we have moved since the last time.
    dx = X - m_DragStartX
    dy = Y - m_DragStartY
    m_DragStartX = X
    m_DragStartY = Y

    ' Move the vertices.
    For i = 1 To m_NumVertices
        With m_Vertices(i)
            .X = .X + dx
            .Y = .Y + dy
        End With
    Next i

    ' Redraw.
    Draw m_DragPolygonCanvas
End Sub

' Stop dragging the vertex.
Private Sub m_DragPolygonCanvas_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    m_DragPolygonCanvas.DrawMode = vbCopyPen
    Set m_DragPolygonCanvas = Nothing

    RaiseEvent PolygonMoved
End Sub


' Continue dragging the vertex.
Private Sub m_DragVertexCanvas_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    ' Erase the previous invert mode image.
    Draw m_DragVertexCanvas

    ' Move the handle.
    With m_Vertices(m_OverPoint)
        .X = X
        .Y = Y
    End With

    ' Redraw.
    Draw m_DragVertexCanvas
End Sub


' Stop dragging the vertex.
Private Sub m_DragVertexCanvas_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    m_DragVertexCanvas.DrawMode = vbCopyPen
    Set m_DragVertexCanvas = Nothing

    RaiseEvent VertexMoved
End Sub


